from git_cloner.model.owner.owner import Owner
from git_cloner.model.repository import RawRepository, Repository
from git_cloner.utils.config import Config
from git_cloner.utils import utils


class GitlabOwner(Owner):
    REPOSITORY_TYPE = "gitlab"

    NAMESPACE_ENDPOINT = "{host}/api/v4/namespaces?search={owner}"
    REPOS_USER_ENDPOINT = "{host}/api/v4/users/{ns}/projects"
    REPOS_GROUPS_ENDPOINT = "{host}/api/v4/groups/{ns}/projects"

    def __init__(self, owner, is_user, host):
        self.host = host
        self.owner = owner
        self.is_user = is_user
        self.namespace = self._get_ns(owner, host)

    @classmethod
    def _get_ns(cls, owner, host):
        results = utils.make_request(cls.NAMESPACE_ENDPOINT.format(host=host, owner=owner))
        raw = [result for result in results if result['name'] == owner][0]
        return raw['id']

    @staticmethod
    def build_from_config():
        conf = Config.get_config()
        owner, is_user = (conf.target_user, True) if conf.target_user else (conf.org, False)
        host = conf.host
        return GitlabOwner(owner, is_user, host)

    def get_repos(self):
        url = self.REPOS_USER_ENDPOINT if self.is_user else self.REPOS_GROUPS_ENDPOINT
        result = utils.make_request(url.format(host=self.host, ns=self.namespace))
        return [RawRepository.from_raw_data(repo, Repository.GITLAB) for repo in result]
