from uuid import uuid4

from git_cloner.model.remotes.remote import Remote
import re


class GitlabRemote(Remote):
    SSH_URL_BASE = "ssh://git@{host}/{owner}/{repo}"
    HTTP_URL_BASE = "{host}/{owner}/{repo}"

    def __init__(self, host, owner, repo_name, is_ssh, port, name=uuid4()):
        self.port = port
        self.name = name
        self.host = host
        self.owner = owner
        self.repo_name = repo_name
        self.is_ssh = is_ssh

    def get_name(self):
        return self.name

    def get_host(self):
        return self.host

    def get_owner(self):
        return self.owner

    def build_remote_url(self):
        if self.is_ssh:
            split = self.host.split("//")[1].split(":", 1)
            new_host = "{}:{}{}".format(
                split[0],
                self.port,
                re.sub('^[\d]*', '', "" if len(split) == 1 else split[1]))
            return self.SSH_URL_BASE.format(host=new_host, owner=self.owner, repo=self.repo_name, port=self.port)
        else:
            return self.HTTP_URL_BASE.format(host=self.host, owner=self.owner, repo=self.repo_name)
