import logging

from git_cloner.model.remotes.github_remote import GithubRemote
from git_cloner.model.remotes.gitlab_remote import GitlabRemote
from git_cloner.utils.config import Config


def get_logger_console(origin_module,
                       str_format='%(asctime)s %(name)-12s %(levelname)-8s %(message)s',
                       date_format='%m-%d %H:%M'):
    level = logging.DEBUG if Config.get_config().verbose else logging.INFO
    logging.basicConfig(
        level=level,
        format=str_format,
        datefmt=date_format)
    logger_name = origin_module if isinstance(origin_module, str) else origin_module.__module__
    return logging.getLogger(logger_name)


def log_result(remote, operation, result):
    template = "\nServer: {}, Repo: {}/{}, Operation: {}, Result: {}"
    config = Config.get_config()
    if config.save is not None:
        with open(Config.get_config().save, "a") as my_file:
            my_file.write(template.format(config.type, remote.owner, remote.repo_name, operation, result))


def build_remote_from_log_line(line):
    server = line.split(",")[0].split(":")[1].strip()
    owner = line.split(",")[1].split(":")[1].split("/")[0].strip()
    name = line.split(",")[1].split(":")[1].split("/")[1].strip()
    config = Config.get_config()
    from git_cloner.model.repository import Repository
    if server == Repository.GITHUB: return GithubRemote(owner, name, config.ssh)
    if server == Repository.GITLAB: return GitlabRemote(config.host,owner, name, config.ssh)
    raise RuntimeError("Invalid repository type")
