import unittest

import os
from pkg_resources import resource_filename

from git_cloner.model.owner.gitlab_owner import GitlabOwner
from git_cloner.model.owner.owner import Owner
from git_cloner.model.repository import Repository
from git_cloner.utils.config import Config
from git_cloner.utils.utils import read

secrets_path = os.path.join(resource_filename('tests', 'resources'), 'secrets')
user = read(os.path.join(secrets_path, 'user'))
host = read(os.path.join(secrets_path, 'gitlab_host'))
org = read(os.path.join(secrets_path, 'personal_group_test'))


class GitlabOwnerIT(unittest.TestCase):
    def test_builder_it(self):
        config = Config.load_config({})
        config.token = read(os.path.join(secrets_path, 'token_gitlab'))
        owner = GitlabOwner(user, True, "https://gitlab.com")
        self.assertEqual(owner.namespace, 2130450)

    def test_builder_unauthorized_it(self):
        Config.load_config({})
        self.assertRaises(RuntimeError, GitlabOwner._get_ns, user, "https://gitlab.com")

    def test_builder_personal_it(self):
        config = Config.load_config({})
        config.token = read(os.path.join(secrets_path, 'token_gitlab_personal'))
        owner = GitlabOwner(user, True, host)
        self.assertEqual(owner.namespace, 2)

    def test_build_gitlab_it(self):
        config = Config.load_config({})

        config.host = host
        config.target_user = user
        config.token = read(os.path.join(secrets_path, 'token_gitlab_personal'))

        owner = Owner.builder(Repository.GITLAB)
        self.assertTrue(isinstance(owner, GitlabOwner))
        self.assertEqual(owner.owner, user)
        self.assertEqual(owner.host, host)
        self.assertEqual(owner.namespace, 2)

    def test_get_repos_user_it(self):
        config = Config.load_config({})

        config.host = host
        config.org = org
        config.token = read(os.path.join(secrets_path, 'token_gitlab_personal'))
        owner = Owner.builder(Repository.GITLAB)

        result = owner.get_repos()

        self.assertEqual(len(result), 1)
        print(result)
