import unittest

from mock_decorators.function_mock import FunctionMockResult
from git_cloner.utils import utils
from git_cloner.model.owner.github_owner import GithubOwner
from git_cloner.model.owner.gitlab_owner import GitlabOwner
from git_cloner.model.owner.owner import Owner
from git_cloner.model.repository import Repository
from git_cloner.utils.config import Config


class OwnerTest(unittest.TestCase):
    def test_builder_github(self):
        Config.load_config({}).target_user = "pepe"
        result = Owner.builder(Repository.GITHUB)
        self.assertEqual(result.owner, "pepe")
        self.assertEqual(result.is_user, True)
        self.assertTrue(isinstance(result, GithubOwner))

    @FunctionMockResult(utils, "make_request", [{"name": "user", "id": "1234"}])
    def test_build_gitlab(self):
        config = Config.load_config({})

        config.host = "http://host"
        config.target_user = "user"

        owner = Owner.builder(Repository.GITLAB)
        self.assertTrue(isinstance(owner, GitlabOwner))
        self.assertEqual(owner.owner, "user")
        self.assertEqual(owner.host, "http://host")
        self.assertEqual(owner.namespace, 1234)

    def test_builder_other(self):
        self.assertRaises(RuntimeError, Owner.builder, "invalid")
        pass
