import subprocess
import unittest

from mock_decorators.function_mock import FunctionMock
from mock_decorators.object_mock import ObjectMock

from git_cloner.model.remotes.github_remote import GithubRemote
from git_cloner.model.repository import Repository, RawRepository
from git_cloner.utils.config import Config
from tests.git_cloner.utils.gen_utils import *
from tests.git_cloner.utils.utils import *

test_dir = "/tmp/test_tmp"


class RepositoryIT(unittest.TestCase):
    def setUp(self):
        self.old_dir = set_test_dir(test_dir)

    def tearDown(self):
        # restore_path(self.old_dir)  # , test_dir)
        pass

    # This tests have problems if are executed in parallel
    def test_serials_it(self):
        def creating_with_cloning():
            remote = GithubRemote.build_from_url("https://github.com/fhuertas/scala_base.git")
            repo = Repository.clone(remote, test_dir)

            self.assertTrue(repo.check_git())
            self.assertEqual(len(repo.remotes), 1)
            self.assertEqual(repo.remotes[0].get_name(), "origin")
            self.assertEqual(repo.remotes[0].build_remote_url(), "https://github.com/fhuertas/scala_base.git")

        def creating_with_cloning_ssh():
            remote = GithubRemote("fhuertas", "mock_decorators", True)
            repo = Repository.clone(remote, test_dir)

            self.assertTrue(repo.check_git())
            self.assertEqual(len(repo.remotes), 1)
            self.assertEqual(repo.remotes[0].get_name(), "origin")
            self.assertEqual(repo.remotes[0].build_remote_url(), "git@github.com:fhuertas/mock_decorators.git")

        def clone_repo_github():
            Config.load_config({}).ssh = True
            gen_raw_repo = generate_repo_github("monkey", "fhuertas")
            raw_repo = RawRepository(gen_raw_repo, Repository.GITHUB)
            result = raw_repo.clone(test_dir)
            self.assertTrue(result.check_git())

        def clone_repo_gitlab_ssh():
            Config.load_config({}).ssh = True
            gen_raw_repo = generate_repo_gitlab("test_project", "test_group")
            raw_repo = RawRepository(gen_raw_repo, Repository.GITLAB)
            result = raw_repo.clone(test_dir)
            self.assertTrue(result.check_git())

        # creating_with_cloning()
        # creating_with_cloning_ssh()
        clone_repo_github()
        clone_repo_gitlab_ssh()

    def check_force_remove_dir_it(self):
        remote = GithubRemote.build_from_url("https://github.com/fhuertas/scala_base.git")
        target_path = os.path.join(test_dir, remote.repo_name)
        os.mkdir(target_path)
        Config.load_config({'--force': True})

        def clone_subprocess_function(*args, **kwargs):
            os.makedirs(target_path)
            return_process = ObjectMock
            return_process.returncode = 0
            return return_process

        @FunctionMock(entity=subprocess, function_name="run", mocked_function=clone_subprocess_function,
                      check_signature=False)
        def inner_test():
            return Repository.clone(remote, test_dir)

        result = inner_test()
        self.assertTrue(os.path.exists(target_path))
        self.assertIsNotNone(result)

    def check_no_force_remove_dir_it(self):
        remote = GithubRemote.build_from_url("https://github.com/fhuertas/scala_base.git")
        target_path = os.path.join(test_dir, remote.repo_name)
        os.mkdir(target_path)

        Config.load_config({'--force': False})
        # Config.load_config({'--force': False, '--save': target_path_log})

        result = Repository.clone(remote, test_dir)
        self.assertIsNone(result)
        self.assertTrue(os.path.exists(target_path))
