import subprocess
import unittest
from pathlib import Path

import os
from pip._vendor.distlib._backport import shutil

from git_cloner import structure_repo


class TestStructureRepo(unittest.TestCase):
    _resources_path = "/tmp"
    _tmp_path = os.path.join(_resources_path, "tmp")
    git = ["git_1", "git_2", "git_3"]
    no_git = ["no_git_1", "no_git_2", "no_git_3"]
    files = ["file_1", "file_2", "file_3"]
    git_paths = []
    no_git_paths = []
    files_paths = []

    def setUp(self):
        shutil.rmtree(self._tmp_path, True)
        self.git_paths = [os.path.join(self._tmp_path, name) for name in self.git]
        self.no_git_paths = [os.path.join(self._tmp_path, name) for name in self.no_git]
        self.files_paths = [os.path.join(self._tmp_path, name) for name in self.files]
        self.directory = os.getcwd()
        os.makedirs(self._tmp_path, mode=0o777, exist_ok=True)
        [Path(path).touch() for path in self.files_paths]
        [os.makedirs(path, exist_ok=True) for path in self.git_paths]
        [os.makedirs(path, exist_ok=True) for path in self.no_git_paths]
        cmds = ["git init {}".format(path) for path in self.git_paths]
        [subprocess.run(cmd.split(" "), stdout=subprocess.PIPE, check=True) for cmd in cmds]

    def tearDown(self):
        # shutil.rmtree(self._tmp_path, True)
        os.chdir(self.directory)

    def test_get_repos(self):
        result = set(structure_repo.list_repos(self._tmp_path))
        self.assertTrue(result == set(self.git_paths))

    def test_check_git_dir_false(self):
        [self.assertFalse(structure_repo.check_git_dir(path)) for path in self.no_git_paths]

    def test_check_git_dir_true(self):
        [self.assertTrue(structure_repo.check_git_dir(path)) for path in self.git_paths]
