import getpass
import unittest

from mock_decorators.function_mock import FunctionMockResult

from git_cloner.utils import config
from git_cloner.utils.config import Config


class TestConfig(unittest.TestCase):
    @FunctionMockResult(entity=getpass, function_name="getpass", result="password", checkExists=False)
    @FunctionMockResult(entity=config, function_name="read", result="THIS_IS_A_TOKEN", checkExists=False)
    def test_arguments(self):
        arguments = {
            Config._KEY_USER: "user",
            Config._KEY_ORG: "organization",
            Config._KEY_LIMIT: "100",
            Config._KEY_DIR_OUT: "out",
            Config._KEY_FORCE: True,
            Config._KEY_SAVE: "save",
            Config._KEY_SSH: True,
            Config._KEY_DIR_IN: "in",
            Config._KEY_TOKEN: "token_path",
            Config._KEY_TYPE: "type",
            Config._KEY_HOST: "git.host.com",
            Config._KEY_TARGET_USER: "pepe",
            Config._KEY_VERBOSE: True,
            Config._KEY_RETRY: "/tmp/log"
        }
        my_config = Config.load_config(arguments)

        self.assertEqual(my_config.user, "user")
        self.assertEqual(my_config.password, "password")
        self.assertEqual(my_config.org, "organization")
        self.assertEqual(my_config.limit, 100)
        self.assertEqual(my_config.dir_out, "out")
        self.assertEqual(my_config.dir_in, "in")
        self.assertEqual(my_config.force, True)
        self.assertEqual(my_config.save, "save")
        self.assertEqual(my_config.ssh, True)
        self.assertEqual(my_config.type, "type")
        self.assertEqual(my_config.token, "THIS_IS_A_TOKEN")
        self.assertEqual(my_config.host, "git.host.com")
        self.assertEqual(my_config.target_user, "pepe")
        self.assertEqual(my_config.verbose, True)
        self.assertEqual(my_config.retry, "/tmp/log")

    def test_no_arguments(self):
        arguments = {}
        my_config = Config.load_config(arguments)
        self.assertEqual(my_config.user, None)
        self.assertEqual(my_config.password, None)
        self.assertEqual(my_config.org, None)
        self.assertEqual(my_config.limit, 100)
        self.assertEqual(my_config.dir_out, ".")
        self.assertEqual(my_config.dir_in, ".")
        self.assertEqual(my_config.force, False)
        self.assertEqual(my_config.save, None)
        self.assertEqual(my_config.ssh, False)
        self.assertEqual(my_config.type, None)
        self.assertEqual(my_config.token, None)
        self.assertEqual(my_config.host, None)
        self.assertEqual(my_config.target_user, None)
        self.assertEqual(my_config.verbose, False)
        self.assertEqual(my_config.retry, None)
        self.assertEqual(my_config.port, 22)
